#define TEST_PAM_STACK "login"
#define TEST_PAM_USER "root"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <security/pam_appl.h>

pam_handle_t *pamh = NULL;
int conv_called = 0;
static char *current_func;

static int 
my_conv1(int n, const struct pam_message **msg,
    struct pam_response **resp, void *data)
{
	conv_called = 1;
	printf("%s conversation %s called\n", current_func, __func__);
	return (PAM_CONV_ERR);
}
static struct pam_conv conv1 = { my_conv1, NULL };

static int 
my_conv2(int n, const struct pam_message **msg,
    struct pam_response **resp, void *data)
{
	conv_called = 2;
	printf("%s conversation %s called\n", current_func, __func__);
	return (PAM_CONV_ERR);
}
static struct pam_conv conv2 = { my_conv2, NULL };

void
prt_result(int result)
{
	printf("%s result %d (%s)\n",current_func, result,
	    pam_strerror(pamh, result));
}

int main(int argc, char *argv[])
{
	int result;

	current_func = "pam_start";  conv_called = 0;
	result = pam_start(TEST_PAM_STACK, TEST_PAM_USER, &conv1, &pamh);
	prt_result(result);

	current_func = "pam_authenticate";  conv_called = 0;
	result = pam_authenticate(pamh, 0);
	prt_result(result);

	current_func = "pam_set_item";  conv_called = 0;
	result = pam_set_item(pamh, PAM_CONV, (const void *)&conv2); 
	prt_result(result);

	current_func = "pam_chauthtok";  conv_called = 0;
	result = pam_chauthtok(pamh, 0);
	prt_result(result);

	switch(conv_called) {
	case 1:
		printf("ERROR: wrong conversation function called by PAM\n");
		break;
	case 2:
		printf("Test passed OK\n");
		break;
	default:
		printf("No conversation called, unable to determine\n");
	}
	exit(0);
}
